

/**********************
*                     *
*  COMPILER INCLUDES  *
*                     *
**********************/


#include <cassert>


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "dnsutil.h"
#include "ex.h"
#include "format.h"


using WWWI::HexDump;
using WWWI::HTON;
using WWWI::NTOH;


/*********************
*                    *
*  METHOD DUMPRANGE  *
*                    *
*********************/


template <class SM> void Buffer::DumpRange(SM &io_smr, off_t i_of, size_t i_sz) const {
  ::HexDump(io_smr,m_ucpBuffer+i_of,i_sz,2,8);
}


/***************
*              *
*  METHOD GET  *
*              *
***************/


inline unsigned char Buffer::Get(off_t i_of) const {
  if (i_of>=(off_t)m_szBuffer) 
    throw BadParseEx(FOUT("Buffer::Get: buffer overrun at ",i_of," max ",m_szBuffer));
  return m_ucpBuffer[i_of];
}


/*******************
*                  *
*  METHODS GETXXX  *
*                  *
*******************/


inline const unsigned char *Buffer::GetBuffer() const { return m_ucpBuffer; }
inline off_t Buffer::GetPos() const { return m_ofPos; }
inline size_t Buffer::GetSize() const { return m_szBuffer; }


/********************
*                   *
*  METHOD PREADNBO  *
*                   *
********************/


template <class T> void Buffer::PReadNBO(T &o_tr, off_t i_off) const {
  this->ReadData(&o_tr,sizeof(T),i_off);
  o_tr = NTOH(o_tr);
}


/*********************
*                    *
*  METHOD PWRITENBO  *
*                    *
*********************/


template <class T> void Buffer::PWriteNBO(const T &ci_tr, off_t i_of) {
  T t = HTON(ci_tr);
  this->WriteData(&t,sizeof(T),i_of);
}


/****************
*               *
*  METHOD READ  *
*               *
****************/


template <class T> void Buffer::Read(T &o_tr) const {
  this->ReadData(&o_tr,sizeof(T),m_ofPos);
}


/*******************
*                  *
*  METHOD READNBO  *
*                  *
*******************/


template <class T> void Buffer::ReadNBO(T &o_tr) const {
  T tTemp;
  this->ReadData(&tTemp,sizeof(T),m_ofPos);
  o_tr = NTOH(tTemp);
}


/****************
*               *
*  SET METHODS  *
*               *
****************/


inline void Buffer::SetPos(off_t i_ofPos) { assert(i_ofPos<(off_t)m_szBufferMax); m_ofPos = i_ofPos; }


/********************
*                   *
*  METHOD TRUNCATE  *
*                   *
********************/


inline void Buffer::Truncate() { m_szBuffer = (size_t)m_ofPos; }


/*****************
*                *
*  METHOD WRITE  *
*                *
*****************/


template <class T> void Buffer::Write(const T &ci_tr) {
  return this->WriteData(&ci_tr,sizeof(T));
}


/********************
*                   *
*  METHOD WRITENBO  *
*                   *
********************/


template <class T> void Buffer::WriteNBO(const T &ci_tr) {
  T t = HTON(ci_tr);
  this->WriteData(&t,sizeof(T));
}


