// $Id: dnsutil.i,v 1.22 2001/11/05 03:26:18 jdw Exp $


/**********************
*                     *
*  COMPILER INCLUDES  *
*                     *
**********************/


#include <cassert>
#include <cstring>
#include <stdexcept>


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "debugptr.h"
#include "loghandle.h"
#include "ex.h"


using std::invalid_argument;
using WWWI::IPDump;


/***********************
*                      *
*  FUNCTIONS FROMBITS  *
*                      *
***********************/


inline unsigned char FromBits(bool i_b3, bool i_b2, bool i_b1, bool i_b0) {
  return FromBits(false,false,false,false,i_b3,i_b2,i_b1,i_b0);
}


inline unsigned char FromBits(bool i_b7, bool i_b6, bool i_b5, bool i_b4, bool i_b3, bool i_b2, bool i_b1, bool i_b0) {
  unsigned char uc = 0;
  if (i_b7) uc+=128;
  if (i_b6) uc+=64;
  if (i_b5) uc+=32;
  if (i_b4) uc+=16;
  if (i_b3) uc+=8;
  if (i_b2) uc+=4;
  if (i_b1) uc+=2;
  if (i_b0) uc+=1;
  return uc;
}


/************************
*                       *
*  FUNCTION MATCHCLASS  *
*                       *
************************/


inline bool MatchClass(RRClass i_cl1, RRClass i_cl2) {
  if (i_cl2==CL_QANY) return true;
  return (i_cl1==i_cl2);
}


/***********************
*                      *
*  FUNCTION MATCHTYPE  *
*                      *
***********************/


// STUB: there are more wildcards that should be handled here.
inline bool MatchType(RRType i_ty1, RRType i_ty2) {
  if (i_ty2==TY_QANY) return true;
  return (i_ty1==i_ty2);
}


/************************
*                       *
*  FUNCTION SOFTDELETE  *
*                       *
************************/


template <class T> void SoftDelete(T* &io_tpr) {
  if (io_tpr==NULL) return;
  delete io_tpr;
  io_tpr = NULL;
}


#ifdef USE_DEBUG_POINTERS
template <class T> void SoftDelete(DebugPtr<T> &io_tdp) {
  if (io_tdp==NULL) return;
  io_tdp.Delete();
}
#endif


/*****************************
*                            *
*  FUNCTION SOFTDELETEARRAY  *
*                            *
*****************************/


template <class T> void SoftDeleteArray(T* &io_tpr) {
  if (io_tpr==NULL) return;
  delete[] io_tpr;
  io_tpr = NULL;
}


