

/*************
*            *
*  SENTINEL  *
*            *
*************/


#ifndef WWWI_MIND_FORMAT_H
#define WWWI_MIND_FORMAT_H


/**********************
*                     *
*  COMPILER INCLUDES  *
*                     *
**********************/


#include <sstream>


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


using std::stringstream;
using std::string;


/*****************
*                *
*  CLASS FORMAT  *
*                *
*****************/

/*
 *  The format class takes a series of  as arguments to its constructor 
 *  and has a cast to const char *.  This means you can use it in a function call
 *  that takes a const char * parameter and the string will be valid for the life
 *  of the call (and no longer).  I split this into two classes to avoid any
 *  surprise implicit casts.
 *
 *  For Example:
 *
 *  throw tantrum(FOUT("Couldn't alloc ",iBytes," bytes at ",__FILE__,__LINE__));
 *
 *  WARNING: Use of these classes is an evil act and may cause your alignment 
 *           to shift.
 *
 */

class FIN {
public:
                      FIN(void) { m_bValid = false; };
  template <class T>  FIN(const T &ci_tr) { 
                        stringstream ss; 
                        ss << ci_tr; 
                        m_xtr = ss.str(); 
                        m_bValid = true;
                      };
  inline bool         GetValid(void) const { return m_bValid; } 
  inline const char  *Out(void) const { return m_xtr.c_str(); }

protected:
  bool m_bValid;
  string m_xtr;
};

static FIN NOFIN;

class FOUT {
public:
   FOUT(const FIN &ci_fm1 = NOFIN,
        const FIN &cm_fm2 = NOFIN,
        const FIN &cm_fm3 = NOFIN,
        const FIN &cm_fm4 = NOFIN,
        const FIN &cm_fm5 = NOFIN,
        const FIN &cm_fm6 = NOFIN,
        const FIN &cm_fm7 = NOFIN,
        const FIN &cm_fm8 = NOFIN,
        const FIN &cm_fm9 = NOFIN,
        const FIN &cm_fm10 = NOFIN);
   operator const char *(void) const { return m_xtr.c_str(); };

protected:
  string m_xtr;
};


/************
*           *
*  THE END  *
*           *
************/


#endif


