

/**********************
*                     *
*  COMPILER INCLUDES  *
*                     *
**********************/


#include <cassert>


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "loghandle.h"
#include "wwwi/singleton.h"
#include "wwwi/string.h"


using std::strchr;
using std::string;
using WWWI::GetSingleton;
using WWWI::StringDuplicate;


/**************************
*                         *
*  LOGHANDLE CONSTRUCTOR  *
*                         *
**************************/


LogHandle::LogHandle(LogFacility i_lf, LogPriority i_ly) {
  m_lf = i_lf;
  m_ly = i_ly;
  m_lyBase = i_ly;
}


/****************
*               *
*  OPERATOR ()  *
*               *
****************/


void LogHandle::operator()() {
  this->Flush();
  m_ly = m_lyBase;
}


/****************
*               *
*  OPERATOR <<  *
*               *
****************/


LogHandle &LogHandle::operator<<(LogPriority i_ly) {
  if (i_ly!=m_ly) {
    this->Flush();
    m_ly = i_ly;
  }
  return *this;
}


/*****************
*                *
*  METHOD FLUSH  *
*                *
*****************/


void LogHandle::Flush() {
  char *strBuffer;
  char *strEnd;
  char *strPos;
  string xtrLine;
  MINDLogPtr lgp = GetSingleton<MINDLog>();

  strBuffer = StringDuplicate(this->str().c_str());
  this->str(string());
  this->clear();

  strPos = strBuffer;
  while ((strEnd = strchr(strPos,'\n'))!=NULL) {
    *strEnd++ = 0;
    lgp->Log(m_lf,m_ly,strPos);
    strPos = strEnd;
  }
  if (*strPos!=0) lgp->Log(m_lf,m_ly,strPos);
  delete[] strBuffer;
}


/*************************
*                        *
*  LOGHANDLE DESTRUCTOR  *
*                        * 
*************************/


LogHandle::~LogHandle() {
  this->Flush();
}



