

/*************
*            *
*  SENTINEL  *
*            *
*************/


#ifndef WWWI_MIND_MIND_H
#define WWWI_MIND_MIND_H


/**********************
*                     *
*  COMPILER INCLUDES  *
*                     *
**********************/


#include "config.h"
#ifdef HAVE_OSTREAM
#include <ostream>
#endif
#ifdef HAVE_OSTREAM_H
#include <ostream.h>
#endif


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "wwwi/types.h"


#ifdef USE_DEBUG_POINTERS
#include "debugptr.h"
#endif


using std::endl;
using std::ostream;
using WWWI::IPAddress;


/************
*           *
*  DEFINES  *
*           *
************/


#ifdef USE_DEBUG_POINTERS

#define DECL_POINTER_TYPES(CLASS) typedef DebugPtr<CLASS> CLASS##Ptr; \
                                  typedef DebugPtr<const CLASS> CLASS##ConstPtr; 

#else 

#define DECL_POINTER_TYPES(CLASS) typedef CLASS *CLASS##Ptr; \
                                  typedef const CLASS *CLASS##ConstPtr;

#endif


/**********************
*                     *
*  TYPE DECLARATIONS  *
*                     *
**********************/

/*
 *  MIND supports QUERY opcodes only.
 */

typedef enum { OPCODE_QUERY=0, 		// RFC1035
               OPCODE_IQUERY=1, 	// RFC1035 (hopefully soon obsolete)
               OPCODE_STATUS=2,		// RFC1035 (undefined?)
               OPCODE_NOTIFY=4,		// RFC1996
               OPCODE_UPDATE=5		// RFC2136
} Opcode;


typedef enum { RCODE_NOERROR=0,		// RFC1035: No Error
               RCODE_FORMERR=1,		// RFC1035: Format Error
               RCODE_SERVFAIL=2,	// RFC1035: Server failure
               RCODE_NXDOMAIN=3,	// RFC1035: Non-existent domain
               RCODE_NOTIMP=4,		// RFC1035: Not implemented
               RCODE_REFUSED=5,		// RFC1035: Query refused
               RCODE_YXDOMAIN=6,	// RFC2136: Name exists when it should not
               RCODE_YXRRSET=7,		// RFC2136: RRSet exists when it should not
               RCODE_NXRRSET=8,		// RFC2136: RRSet that should exist does not
               RCODE_NOTAUTH=9,		// RFC2136: Server not authoritative for zone
               RCODE_NOTZONE=10		// RFC2136: Name not contained in zone
} RCode;


/*
 *  MIND supports only the IN class.
 */
enum rr_class_enum { CL_IN=1, 
                     CL_CS=2, 
                     CL_CH=3, 
                     CL_HS=4, 
                     CL_QANY=255 };
typedef enum rr_class_enum RRClass;


/*
 *   These are the types of RR records.  Support is indicated by the RWC 
 *   characters, which stand for Yes or No.  R means the type will be read 
 *   with compression.  W means the type will be written with compression.  
 *   C means the type may be cached.
 */
//                                         RWC RFC 
typedef enum { TY_A=1, 			// NNY RFC1035 (AddressLens)
               TY_NS=2, 		// YYY RFC1035 (NameLens)
               TY_MD=3, 		// YYY RFC1035 OBSOLETE (NameLens)
               TY_MF=4, 		// YYY RFC1035 OBSOLETE (NameLens)
               TY_CNAME=5,		// YYY RFC1035 (NameLens)
               TY_SOA=6, 		// YYY RFC1035 (SOALens)
               TY_MB=7, 		// YYY RFC1035 EXPERIMENTAL (NameLens)
               TY_MG=8, 		// YYY RFC1035 EXPERIMENTAL (NameLens)
               TY_MR=9, 		// YYY RFC1035 EXPERIMENTAL (NameLens)
               TY_NULL=10,		// NNY RFC1035 EXPERIMENTAL (RawLens)
               TY_WKS=11, 		// NNY RFC1035 (RawLens)
               TY_PTR=12, 		// YYY RFC1035 (NameLens)
               TY_HINFO=13, 		// NNY RFC1035 (RawLens)
               TY_MINFO=14,		// NNN RFC1035 EXPERIMENTAL (Not Supported)
               TY_MX=15, 		// YYY RFC1035 (MXLens)
               TY_TXT=16, 		// NNY RFC1035 (RawLens)
               TY_AAAA=28,              // NNN IANA (RawLens)
               TY_OPT=41,		// NNN RFC2671 (Not Supported)

               TY_QIXFR=251,            //     RFC1995 (Not Supported)
               TY_QAXFR=252, 		//     RFC1035 (Not Supported)
               TY_QMAILB=253,		//     RFC1035 EXPERIMENTAL (Not Supported)
               TY_QMAILA=254, 		//     RFC1035 OBSOLETE (Not Supported)
               TY_QANY=255		//     RFC1035
} RRType;


/************
*           *
*  THE END  *
*           *
************/


#endif


