

/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "ex.h"
#include "rdata_mx.h"


/************************
*                       *
*  RDATAMX CONSTRUCTOR  *
*                       *
************************/


RDataMX::RDataMX() {
  m_llp = NULL;
}


/*****************
*                *
*  METHOD CLONE  *
*                *
*****************/


RDataPtr RDataMX::Clone() const {
  RDataMX *rdpOut = new RDataMX;
  rdpOut->m_llp = new LabelList(*m_llp);
  return (RDataPtr)rdpOut;
}


/*******************
*                  *
*  METHOD COMPARE  *
*                  *
*******************/


bool RDataMX::Compare(const RData &ci_rdr) const {
  const RDataMX *c_rdp = ci_rdr.As<RDataMX>();
  if (c_rdp->m_us!=m_us) return false;
  return (*(c_rdp->m_llp)==(*m_llp));
}

 
/****************
*               *
*  METHOD DUMP  *
*               *
****************/


void RDataMX::Dump(ostream &io_smr) const {
  io_smr << m_us << " " << m_llp;
}


/*******************
*                  *
*  METHOD ISMATCH  *
*                  *
*******************/


bool RDataMX::IsMatch(RRType i_ty, RRClass i_cl) {
  // RFC 1035 says this is class-independent.
  return (i_ty==TY_MX);
}


/*********************
*                    *
*  METHOD READRDATA  *
*                    *
*********************/


void RDataMX::ReadRData(BufferConstPtr ci_dpp) {
  unsigned short usLength;

  ci_dpp->ReadNBO(usLength);
  ci_dpp->ReadNBO(m_us);
  ci_dpp->ReadLabelList(m_llp,true);
}


/********************
*                   *
*  METHOD SETRDATA  *
*                   *
********************/


void RDataMX::SetRData(const char *ci_strRData) {
  EX_MIND("RDataMX::SetRData: I was too lazy too implement this.");
}


/**********************
*                     *
*  METHOD WRITERDATA  *
*                     *
**********************/


void RDataMX::WriteRData(BufferPtr io_dpp) const {
  unsigned short usLength;
  off_t ofPos;

  ofPos = io_dpp->GetPos();
  io_dpp->WriteNBO((unsigned short)0);
  io_dpp->WriteNBO(m_us);
  usLength = io_dpp->WriteLabelList(m_llp,true) + sizeof(m_us);
  io_dpp->PWriteNBO(usLength,ofPos);
}


/***********************
*                      *
*  RDATAMX DESTRUCTOR  *
*                      *
***********************/


RDataMX::~RDataMX() {
  SoftDelete(m_llp);
}


