

/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "rdata_name.h"


/**************************
*                         *
*  RDATANAME CONSTRUCTOR  *
*                         *
**************************/


RDataName::RDataName() {
  m_llp = NULL;
}


/*****************
*                *
*  METHOD CLONE  *
*                *
*****************/


RDataPtr RDataName::Clone() const {
  RDataName *rdpOut = new RDataName;
  rdpOut->m_llp = new LabelList(*m_llp);
  return (RDataPtr)rdpOut;
}


/*******************
*                  *
*  METHOD COMPARE  *
*                  *
*******************/


bool RDataName::Compare(const RData &ci_rdr) const {
  LabelListConstPtr c_llpFar = ci_rdr.As<RDataName>()->GetRDName();
  return ((*c_llpFar)==(*m_llp));
}

 
/****************
*               *
*  METHOD DUMP  *
*               *
****************/


void RDataName::Dump(ostream &io_smr) const {
  io_smr << this->GetRDName();
}


/*********************
*                    *
*  METHOD GETRDNAME  *
*                    *
*********************/


LabelListConstPtr RDataName::GetRDName() const { return m_llp; }


/*******************
*                  *
*  METHOD ISMATCH  *
*                  *
*******************/


bool RDataName::IsMatch(RRType i_ty, RRClass i_cl) {
  switch(i_ty) {
//  RFC 1035 "Well Known" types that do not require class IN
  case TY_NS:
  case TY_MD:
  case TY_MF:
  case TY_CNAME:
  case TY_MB:
  case TY_MG:
  case TY_MR:
  case TY_PTR:
    return true;
  default: break;
  }
// If there were any IN-specific name types, they would go here.
  return false;
}           


/*********************
*                    *
*  METHOD READRDATA  *
*                    *
*********************/


void RDataName::ReadRData(BufferConstPtr ci_dpp) {
  unsigned short us;

  ci_dpp->ReadNBO(us);
  ci_dpp->ReadLabelList(m_llp,true);
}


/********************
*                   *
*  METHOD SETRDATA  *
*                   *
********************/


void RDataName::SetRData(const char *ci_strRData) {
  m_llp = new LabelList(ci_strRData);
}


/**********************
*                     *
*  METHOD WRITERDATA  *
*                     *
**********************/


void RDataName::WriteRData(BufferPtr io_dpp) const {
  unsigned short us;
  off_t ofPos;

  ofPos = io_dpp->GetPos();
  io_dpp->WriteNBO((unsigned short)0);
  us = io_dpp->WriteLabelList(m_llp,true);
  io_dpp->PWriteNBO(us,ofPos);
}


/*************************
*                        *
*  RDATANAME DESTRUCTOR  *
*                        *
*************************/


RDataName::~RDataName() {
  SoftDelete(m_llp);
}


