

/*************
*            *
*  SENTINEL  *
*            *
*************/


#ifndef WWWI_MIND_RESOLVER_H
#define WWWI_MIND_RESOLVER_H


/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "mind.h"
#include "dnssocket.h"
#include "question.h"
#include "response.h"
#include "rrcache.h"


/**********************
*                     *
*  TYPE DECLARATIONS  *
*                     *
**********************/


class RRCache;
typedef RRCache* RRCachePtr;


/*******************
*                  *
*  CLASS RESOLVER  *
*                  *
*******************/


/*
 *  The Resolver class performs all the real work associated with resolving
 *  an unknown DNS name.
 *
 *  Properties
 *
 *    RRCachePtr m_cap
 *      This property points to the global cache of RRs.
 *
 *  Methods
 *
 */

class Resolver {
public:
                    Resolver(void);
  ResponsePtr       QueryAuthority(QuestionConstPtr ci_qnp, DNSSocketPtr i_dspAuthority, IPAddress i_ipAuthority);
  ResponsePtr       QueryRecursive(const char *i_strName, RRType i_ty, RRClass i_cl, unsigned i_uSteps);
  ResponsePtr       QueryRecursive(QuestionConstPtr ci_qnp, unsigned i_uSteps);
  inline IPAddress  SimpleResolve(const char *i_strName, unsigned i_uSteps);
  inline IPAddress  SimpleResolve(LabelListConstPtr i_llpName, unsigned i_uSteps);

protected:
  RRCachePtr     m_cap;
  unsigned short m_usID;   // STUB: should go to random to stop prediction

};
typedef Resolver* ResolverPtr;


/*********************
*                    *
*  INLINE FUNCTIONS  *
*                    *
*********************/


/************
*           *
*  THE END  *
*           *
************/


#endif


