

/*********************
*                    *
*  PROJECT INCLUDES  *
*                    *
*********************/


#include "wwwi/mutexholder.h"


using WWWI::MutexHolder;


/****************
*               *
*  METHODS GET  *
*               *
****************/


inline ResponsePtr RRCache::Get(const char *i_strName, RRType i_rrType, RRClass i_rrClass) {
  LabelListPtr llp = new LabelList(i_strName);
  ResponsePtr rsp;
  rsp = this->Get(llp,i_rrType,i_rrClass);
  SoftDelete(llp);
  return rsp;
}


inline ResponsePtr RRCache::Get(QuestionConstPtr ci_qnp) {
  return this->Get(ci_qnp->GetName(),ci_qnp->GetType(),ci_qnp->GetClass());
}


/*********************
*                    *
*  METHOD GETSIMPLE  *
*                    *
*********************/


inline RRListPtr RRCache::GetSimple(const char *i_strName, RRType i_rrType, RRClass i_rrClass) {
  LabelListPtr llp = new LabelList(i_strName);
  RRListPtr rlp;
  rlp = this->GetSimple(llp,i_rrType,i_rrClass);
  SoftDelete(llp);
  return rlp;
}


/*************************
*                        *
*  METHOD SETMAXENTRIES  *
*                        *
*************************/


inline void RRCache::SetMaxEntries(unsigned i_uMaxEntries) { 
  LogHandle lh(LF_CACHE,LY_INFO);
  lh << "cache maxentries set to " << i_uMaxEntries << endl;
  m_uMaxEntries = i_uMaxEntries; 
}


/*************************
*                        *
*  METHOD SETNEXTEXPIRE  *
*                        *
*************************/


inline void RRCache::SetNextExpire(time_t i_tmWhenExpire) {
  MutexHolder mh(&m_mxExpire);
  m_tmNextExpire = i_tmWhenExpire;
}


/*********************
*                    *
*  METHOD TRYEXPIRE  *
*                    *
*********************/


inline void RRCache::TryExpire() {
  time_t tmNow;
  if (m_mxExpire.TryLock()==false) return;
  if (m_bExpiring==false) {
    m_bExpiring = true; 
    m_mxExpire.Unlock();
    tmNow = time(NULL);
    if (tmNow>=m_tmNextExpire) this->Expire();
    m_mxExpire.Lock();
    m_bExpiring = false; 
  }
  m_mxExpire.Unlock();
}


