

#include "tests.h"
#include "../dnspacket.h"
#include "../dnsutil.h"
#include "wwwi/string.h"


using namespace WWWI;


MODULE(DNSPacketTest)
  DNSPacket dp;

  LabelList ll("a.root-servers.net");
  LabelListPtr llp;
  dp.SetPos(24);
  dp.WriteLabelList(&ll,true);
  TEST("WriteLabelList",dp.GetPos(),44);
  dp.SetPos(24);
  dp.ReadLabelList(llp,true);
  TEST("ReadLabelList",dp.GetPos(),44);
  TEST("DNSPacket ll",ll,*llp);
  SoftDelete(llp);

  unsigned char ucaPacket[512];
  unsigned char ucaPacket2[512];
  unsigned short usID;
  RequestPtr rqp;
  ResponsePtr rsp;
  llp = new LabelList("www.wwwi.com");
  
  StringFromHex((char*)ucaPacket,168,"fdcd 8580 0001 0001 0003 0003 0377 7777 "
                                     "0477 7777 6903 636f 6d00 0001 0001 c00c "
                                     "0001 0001 0000 3840 0004 c701 5c1e c010 "
                                     "0002 0001 0000 3840 000e 0770 7269 6d61 "
                                     "7279 0364 6e73 c010 c010 0002 0001 0000 "
                                     "3840 000c 0973 6563 6f6e 6461 7279 c042 "
                                     "c010 0002 0001 0000 3840 000c 036e 7331 "
                                     "0567 6961 636f c015 c06c 0001 0001 0001 "
                                     "5180 0004 8c63 0302 c03a 0001 0001 0000 "
                                     "3840 0004 c701 5c0d c054 0001 0001 0000 "
                                     "3840 0004 c701 5c0a");

  dp.SetPos(0);
  dp.WriteData(ucaPacket,168);
  TEST("WriteData pos",dp.GetPos(),168);
  TEST("WriteData size",dp.GetSize(),168);
  TEST("TestID true",dp.TestID(0xfdcd),true);
  TEST("TestID false",dp.TestID(0x0123),false);
  dp.Parse(usID,rqp,rsp);
  TEST("Parse id",usID,0xfdcd);
  TEST("Parse request rd",rqp->GetRD(),true);
  TEST("Parse request op",rqp->GetOpcode(),OPCODE_QUERY);
  TEST("Parse request question name",*rqp->GetQuestion()->GetName(),*llp);
  TEST("Parse request question type",rqp->GetQuestion()->GetType(),TY_A);
  TEST("Parse request question class",rqp->GetQuestion()->GetClass(),CL_IN);
  TEST("Parse response aa",rsp->GetAA(),true);
  TEST("Parse response tc",rsp->GetTC(),false);
  TEST("Parse response rcode",rsp->GetRCode(),RCODE_NOERROR);
  TEST("Parse response ancount",rsp->GetANCount(),1);
  TEST("Parse response nscount",rsp->GetNSCount(),3);
  TEST("Parse response arcount",rsp->GetARCount(),3);
  SoftDelete(llp);

  cout << rsp;
  
  dp.Format(usID,rqp,rsp);
  dp.SetPos(0);
  dp.ReadData(ucaPacket2,dp.GetSize());
  TEST("ReadData pos",dp.GetPos(),168);
  TEST("Format",memcmp(ucaPacket,ucaPacket2,168),0);
  delete rqp;
  SoftDelete(rsp);

  HexDump(cout,ucaPacket,168,2,8);
  cout << endl;
  HexDump(cout,ucaPacket2,dp.GetSize(),2,8);

ENDMODULE


