

#include "tests.h"
#include "../loghandle.h"
#include "wwwi/singleton.h"


using std::string;
using WWWI::GetSingleton;


MODULE(LogTest)
  LogHandle *lhp;
  MINDLogPtr lgp = GetSingleton<MINDLog>();
  stringstream ss;
  string xtr;

  lgp->SetStream(&ss);

  lhp = new LogHandle(LF_MISC,LY_INFO);
  (*lhp) << "Hello" << " there " << 1 << endl;
  delete lhp;
  xtr = ss.str();
  TEST_NE("Log info",xtr.find("Hello there 1"),string::npos);
  TEST("Log sanity",xtr.find("No such text"),string::npos);

  lhp = new LogHandle(LF_MISC,LY_INFO);
  (*lhp) << "Hello" << " error " << 2 << endl;
  delete lhp;
  xtr = ss.str();
  TEST_NE("Log error",xtr.find("Hello error 2"),string::npos);
  
  lhp = new LogHandle(LF_MISC,LY_DEBUG);
  (*lhp) << "Hello" << " debug " << 3 << endl;
  (*lhp) << LY_EXTRA_DEBUG << "Hello" << " extra " << 4 << endl; (*lhp)();
  delete lhp;
  xtr = ss.str();
  TEST_NE("Log debug",xtr.find("Hello debug 3"),string::npos);
  TEST("Log debug",xtr.find("Hello extra 4"),string::npos);
  
  lgp->SetPriority(LY_INFO);
  lhp = new LogHandle(LF_MISC,LY_INFO);
  (*lhp) << "Hello" << " info " << 5 << endl;
  (*lhp) << LY_DEBUG << "Hello" << " debug " << 6 << endl; (*lhp)();
  (*lhp) << "Hello" << " info " << 7 << endl;
  delete lhp;
  xtr = ss.str();
  TEST_NE("Log include",xtr.find("Hello info 5"),string::npos);
  TEST("Log exclude",xtr.find("Hello debug 6"),string::npos);
  TEST_NE("Log include",xtr.find("Hello info 7"),string::npos);
  cout << xtr;

  lgp->SetStream(&std::cerr);

ENDMODULE


